#! /usr/local/bin/perl

#############################################################################
#  
#   File:    mailform.pl
#
#   Author:  Tom Scavo <trscavo@syr.edu>
#
#   This perl script processes an URL-encoded message
#   containing any number of keyword-value pairs.
#   An e-mail message is constructed from this input.
#   One of the keywords must be:
#
#     _mailTo
#
#   whose value is the address of the recipient of the e-mail message.
#   Here are some examples of HTML constructs satisfying this requirement:
#
#     <INPUT TYPE=HIDDEN NAME=_mailTo VALUE="trscavo@syr.edu">
#     <INPUT TYPE=HIDDEN NAME=_mailTo VALUE="trscavo@syr.edu (Tom Scavo)">
#     <INPUT TYPE=HIDDEN NAME=_mailTo 
#            VALUE="&quot;Tom Scavo&quot; &lt;trscavo@syr.edu&gt;">
#
#   The following keywords are optional, but processed specially:
#
#     _mailFrom
#     _mailSubject
#     _mailCc
#     _mailBcc
#     _mailConfirmURL
#     _mailArchive
#
#   The purpose of the first four of these keywords should be obvious.
#
#   The keyword '_mailConfirmURL' gives the URL of an HTML page
#   confirming that a message has been sent.  If this keyword is 
#   omitted, a default confirmation page will be returned.
#
#   The keyword '_mailArchive' is used to specify a file archive.
#   For example,
#
#     <INPUT TYPE=HIDDEN NAME=_mailArchive VALUE="mail.txt">
#
#   would cause all messages processed by mailform.pl to be archived
#   in a file called "mail.txt".
#
#   All other keyword-value pairs are sent to the recipient specified
#   in '_mailTo', after which the user is notified whether or not the 
#   mailing was successful.
#
#############################################################################

#############################################################################
#  
#   Initialization
#
#############################################################################

$sendmail   = '/usr/lib/sendmail -t -n';
@keywords   = ('_mailTo', 
               '_mailFrom', 
               '_mailSubject',
               '_mailCc', 
               '_mailBcc',
               '_mailConfirmURL',
               '_mailArchive');
$maintainer = 'trscavo@npac.syr.edu';

#############################################################################
#  
#   Main Routines
#
#############################################################################

&ReadInput;
&ValidateData && 
&SendData;

#############################################################################
#  
#   Subroutines
#
#############################################################################

sub ReadInput {
	local ($key, $val);
	local ($remote_address, @subnet_numbers, $packed_address, $remote_host);
	
	&ReadParse;
	# Assign $IN{'_mailTo'} to $mailTo, etc.:
	while (($key,$val) = each %IN) {
		$key =~ s/^_//;
		${$key} = $val;
	}
	
	# Convert IP address into hostname:
	$remote_address = $ENV{'REMOTE_ADDR'};
	if ( defined($remote_address) ) {
		@subnet_numbers = split(/\./, $remote_address);
		$packed_address = pack("C4", @subnet_numbers);
		($remote_host) = gethostbyaddr($packed_address, 2);
# 		if ( defined($remote_host) ) {
# 			$message_origin = $remote_host;
# 		} else {
# 			$message_origin = $remote_address;
# 		}
		$ORIGIN = defined($remote_host) ? $remote_host : $remote_address;
	} else {
		$ORIGIN = "UNKNOWN";
	}
	
	return 1;
}

# Read and parse the data from the HTML form:
sub ReadParse {
	local ($in, @pairs, $key, $val);
	
	# Note the added else-block for command-line debugging:
	if ( $ENV{'REQUEST_METHOD'} eq "GET" ) {
		$in = $ENV{'QUERY_STRING'};
	} elsif ( $ENV{'REQUEST_METHOD'} eq "POST" ) {
		for ( $i = 0; $i < $ENV{'CONTENT_LENGTH'}; $i++ ) {
			$in .= getc;
		}
	} else {
		$in = <STDIN>;
		print "$in\n";
	}
	
	# Split input string into an array of key-value pairs:
	@pairs = split(/&/, $in);
	
	# Process each key-value pair individually:
	foreach $pair (@pairs) {
		# Convert plus signs to spaces:
		$pair =~ s/\+/ /g;
		# Convert %nn from hex to alphanumeric:
# 		$pair =~ s/%(..)/pack("c", hex($1))/ge;
		$pair =~ s/%(..)/pack("C", hex($1))/ge;
		# Split the key-value pair:
		($key, $val) = split(/=/, $pair);
		if (&IsSpecial($key)) {
			# Update associative array indexed by keyword:
			$IN{$key} = $val;
		} else {
			# Store the keys and values in parallel arrays:
			push(@keys, $key); push(@vals, $val);
		}
	}
	
	return 1; 
}

sub ValidateData {
	if ( defined($mailTo) ) {
		return 1;
	} else {
		$SendErrorMsg = "'_mailto' variable not defined!";
		&SendError;
		return 0;
	}
}

sub SendData {
	local ($body);
	
# 	if ( open(MAIL, "|$sendmail \'$mailTo\'") ) {
	if ( open(MAIL, "|$sendmail") ) {
		print MAIL "To: $mailTo\n";
		print MAIL "From: $mailFrom\n";
		print MAIL "Subject: $mailSubject\n";
		print MAIL "Cc: $mailCc\n";
		print MAIL "Bcc: $mailBcc\n";
# 		print MAIL "X-Remote-Host: $ENV{'REMOTE_ADDR'}\n";
		print MAIL "X-Remote-Host: $ORIGIN\n";
		&BuildBody;
		print MAIL "\n$BODY";
		close(MAIL);
		&UpdateArchive &&
		&SendConfirm;
		return 1;
	} else {
		$SendErrorMsg = "$sendmail could not be opened!";
		&SendError;
		return 0;
	}
}

sub BuildBody {
	local ($key, $val);
	
	$BODY = "";
  while ( @keys ) {
  	$key = shift(@keys); $val = shift(@vals);
		$BODY .= "$key:  $val\n";
	}
	
	return 1;
}

sub UpdateArchive {
	local ($body);
	
	if ( defined($mailArchive) ) {
		if ( open(ARCHIVE, ">>$mailArchive") ) {
			print ARCHIVE "Date:  " . `date`;
			print ARCHIVE "To: $mailTo\n";
			print ARCHIVE "From: $mailFrom\n" if ( defined($mailFrom) );
			print ARCHIVE "Subject: $mailSubject\n" if ( defined($mailSubject) );
			print ARCHIVE "Cc: $mailCc\n" if ( defined($mailCc) );
			print ARCHIVE "Bcc: $mailBcc\n" if ( defined($mailBcc) );
			print ARCHIVE "X-Remote-Host: $ORIGIN\n";
			print ARCHIVE "\n$BODY";
			print ARCHIVE "-------------------------------------------------\n\n";
			close(ARCHIVE);
		} else {
			$SendErrorMsg = "$mailArchive could not be opened!";
			&SendError;
			return 0;
		}
	}
	return 1;
}

sub SendConfirm {
	if ( defined($mailConfirmURL) ) {
		print "Location: $mailConfirmURL\n\n";
		return 1;
	}
	
	print "Content-type: text/html\n\n";

	# Opening HTML statements:
	print "<HTML>\n";
	print "<HEAD>\n";
	print "<TITLE>Mail Form Confirmation Notice</TITLE>\n";
	print "</HEAD>\n";
	print "<BODY>\n\n";
	
	# HTML body:
	print "<H1>Notice!</H1>\n";
	print 'Your message has been sent to', "\n";
	print "<P>$mailTo</P>\n";
	print 'Thank you for using mailform.pl!', "\n";

	# Closing HTML statements:
	print "\n</BODY>\n";
	print "</HTML>\n";
	
	return 1;
}

sub SendError {
	print "Content-type: text/html\n\n";

	# Opening HTML statements:
	print "<HTML>\n";
	print "<HEAD>\n";
	print "<TITLE>Mail Form Error Notice</TITLE>\n";
	print "</HEAD>\n";
	print "<BODY>\n\n";
	
	# HTML body:
	print "<H1>Error!</H1>\n";
	print "$SendErrorMsg<P>\n";
	print 'Please contact', "\n";
	print "<A HREF=\"mailto:$maintainer\">the maintainer of this script</A>.\n";

	# Closing HTML statements:
	print "\n</BODY>\n";
	print "</HTML>\n";
	
	return 1;
}

#############################################################################
#  
#   Utilities
#
#############################################################################

# Test for array membership:
sub IsSpecial {
	local ($special);
	
	$special = 0;
	foreach $keyword (@keywords) {
		if ($_[0] eq $keyword) {
			$special = 1;
			last;
		}
	}
	
	return $special;
}